/*
*********************************************************************************************************
*                                             EXAMPLE CODE
*********************************************************************************************************
* Licensing terms:
*   This file is provided as an example on how to use Micrium products. It has not necessarily been
*   tested under every possible condition and is only offered as a reference, without any guarantee.
*
*   Please feel free to use any application code labeled as 'EXAMPLE CODE' in your application products.
*   Example code may be used as is, in whole or in part, or may be used as a reference only. This file
*   can be modified as required.
*
*   You can find user manuals, API references, release notes and more at: https://doc.micrium.com
*
*   You can contact us at: http://www.micrium.com
*
*   Please help us continue to provide the Embedded community with the finest software available.
*
*   Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                             OS TICK BSP
*
* File : bsp_os.c
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*********************************************************************************************************
*/

#include  <cpu.h>
#include  <lib_def.h>
#include  <os.h>
#include  <bsp_os.h>

#include "em_cmu.h"
#include "em_rtc.h"


/*
*********************************************************************************************************
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*********************************************************************************************************
*/

#ifndef  OS_CFG_DYN_TICK_EN                                     /* Dynamic tick only available for uCOS-III             */
#define  OS_CFG_DYN_TICK_EN      DEF_DISABLED
#endif

#if (OS_CFG_DYN_TICK_EN == DEF_ENABLED)
#define  BSP_OS_LPTIMER_TICK_RATE_HZ          (16384u)
#define  BSP_OS_LPTICK_TO_OSTICK(lptick)    (((lptick) * OSCfg_TickRate_Hz)/BSP_OS_LPTIMER_TICK_RATE_HZ)
#define  BSP_OS_OSTICK_TO_LPTICK(ostick)    (((ostick) * BSP_OS_LPTIMER_TICK_RATE_HZ)/OSCfg_TickRate_Hz)
#endif


/*
*********************************************************************************************************
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*********************************************************************************************************
*/

#if (OS_CFG_DYN_TICK_EN == DEF_ENABLED)
static  OS_TICK  BSP_OS_TicksToGo = 0u;                         /* Stored in OS Tick units.                             */
static  OS_TICK  BSP_OS_LastTick  = 0u;                         /* Stored in OS Tick units.                             */
static  OS_TICK  BSP_OS_UnTick    = 0u;                         /* Stored in OS Tick units.                             */
#endif


/*
*********************************************************************************************************
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*********************************************************************************************************
*/

#if (OS_CFG_DYN_TICK_EN == DEF_ENABLED)
static  void     BSP_OS_LP_TickInit(void);
static  OS_TICK  BSP_OS_LP_TickGet (void);
#endif


/*
*********************************************************************************************************
*********************************************************************************************************
*                                           GLOBAL FUNCTIONS
*********************************************************************************************************
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                    INITIALIZE OS TICK INTERRUPT
*
* Description : Initialize the tick interrupt.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none
*********************************************************************************************************
*/

void  BSP_OSTickInit (void)
{
#if (OS_CFG_DYN_TICK_EN != DEF_ENABLED)
    CPU_INT32U  cpu_freq;
#endif

#if (OS_CFG_DYN_TICK_EN == DEF_ENABLED)
    BSP_OS_LP_TickInit();
#else
    cpu_freq = SystemCoreClockGet();                            /* Determine SysTick reference freq.                    */
    OS_CPU_SysTickInitFreq(cpu_freq);                           /* Init uC/OS periodic time src (SysTick).              */
#endif
}


/*
*********************************************************************************************************
*                                     DISABLE OS TICK INTERRUPT
*
* Description : Disable the tick interrupt.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : none
*********************************************************************************************************
*/

void  BSP_OSTickDisable (void)
{
    SysTick->CTRL = 0x0000000;
}


/*
*********************************************************************************************************
*********************************************************************************************************
**                                       uC/OS-III Low Power Tick
*********************************************************************************************************
*********************************************************************************************************
*/

#if (OS_CFG_DYN_TICK_EN == DEF_ENABLED)

/*
*********************************************************************************************************
*                                          BSP_OS_LP_TickInit()
*
* Description : Initialize the RTC timer to enable dynamic ticking.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : BSP_Tick_Init.
*
*               This function is an INTERNAL uC/OS-III function & MUST NOT be called by application
*               function(s).
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  BSP_OS_LP_TickInit (void)
{
    RTC_Init_TypeDef  rtc_init_cfg;


    BSP_OS_TicksToGo = 0u;
    BSP_OS_LastTick  = 0u;


    CMU_ClockEnable(cmuClock_CORELE, true);                     /* Enable and select RTC oscillator.                    */

    CMU_ClockSelectSet(cmuClock_LFA, cmuSelect_LFXO);           /* Select the low frequency crystal oscillator.         */

    CMU_ClockDivSet(cmuClock_RTC, cmuClkDiv_2);                 /* Set 2 times divider to reduce energy.                */

    CMU_ClockEnable(cmuClock_RTC, true);                        /* Enable clock to RTC module.                          */

    rtc_init_cfg.enable   = false;                              /* Specify the RTC initial configuration.               */
    rtc_init_cfg.debugRun = false;
    rtc_init_cfg.comp0Top = true;

    RTC_Init(&rtc_init_cfg);                                    /* Initialization of RTC.                               */

    RTC_IntDisable(RTC_IFC_COMP0);                              /* Disable interrupt generation from RTC0.              */

    NVIC_SetPriority(RTC_IRQn, 255);                            /* Tick interrupt MUST execute at the lowest int prio.  */

    NVIC_ClearPendingIRQ(RTC_IRQn);                             /* Enable interrupts.                                   */
    NVIC_EnableIRQ(RTC_IRQn);

    RTC_CompareSet(0, 128);                                     /* Set compare register.                                */

    RTC_IntClear(RTC_IFC_COMP0);                                /* Get the RTC started.                                 */
    RTC_IntEnable(RTC_IF_COMP0);
    RTC_Enable(true);
}


/*
*********************************************************************************************************
*                                        RTC_IRQHandler()
*
* Description : Handle the RTC interrupt for the OS LP mode.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : This is an ISR.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  RTC_IRQHandler (void)
{
    CPU_SR_ALLOC();


    CPU_CRITICAL_ENTER();
    OSIntEnter();
    CPU_CRITICAL_EXIT();

    RTC_IntClear(_RTC_IFC_MASK);                                /* Clear the RTC tick interrupt.                        */

    if (BSP_OS_TicksToGo != 0) {
    	RTC_CounterSet(0);
        OSTimeDynTick(BSP_OS_TicksToGo + BSP_OS_UnTick + BSP_OS_LPTICK_TO_OSTICK(RTC_CounterGet()));
        BSP_OS_LastTick += BSP_OS_TicksToGo + BSP_OS_LPTICK_TO_OSTICK(RTC_CounterGet());

        BSP_OS_TicksToGo = 0;
        BSP_OS_UnTick    = 0;
    } else {
                                                                /* No tasks need to be awakened only keep track of time */
        BSP_OS_LastTick += BSP_OS_LPTICK_TO_OSTICK(RTC_CompareGet(0));
        BSP_OS_UnTick   += BSP_OS_LPTICK_TO_OSTICK(RTC_CompareGet(0));
    }

    OSIntExit();
}


/*
*********************************************************************************************************
*                                            BSP_OS_TickGet()
*
* Description : Get the OS Tick Counter as if it was running continuously.
*
* Argument(s) : none.
*
* Return(s)   : The effective OS Tick Counter.
*
* Caller(s)   : OS_TaskBlock, OS_TickListInsertDly and OSTimeGet.
*
*               This function is an INTERNAL uC/OS-III function & MUST NOT be called by application
*               function(s).
*
* Note(s)     : none.
*********************************************************************************************************
*/

OS_TICK  BSP_OS_TickGet (void)
{
    OS_TICK  tick;


    tick = BSP_OS_LastTick + BSP_OS_LPTICK_TO_OSTICK(BSP_OS_LP_TickGet());

    return (tick);
}


/*
*********************************************************************************************************
*                                          BSP_OS_TickNextSet()
*
* Description : Set the number of OS Ticks to wait before calling OSTimeTick.
*
* Argument(s) : ticks       number of OS Ticks to wait.
*
* Return(s)   : Number of effective OS Ticks until next OSTimeTick.
*
* Caller(s)   : OS_TickTask and OS_TickListInsert.
*
*               This function is an INTERNAL uC/OS-III function & MUST NOT be called by application
*               function(s).
*
* Note(s)     : none.
*********************************************************************************************************
*/

OS_TICK  BSP_OS_TickNextSet (OS_TICK  ticks)
{
    OS_TICK     lpticks;

                                                                /* Check if OSTimeTick needs to be called.              */
    if (ticks != (OS_TICK)-1) {
                                                                /* Get tick count in LPTMR units.                       */
        lpticks = BSP_OS_OSTICK_TO_LPTICK(ticks);
                                                                /* Adjust for maximum value.                            */
        if (lpticks > DEF_INT_16U_MAX_VAL) {
            BSP_OS_TicksToGo = BSP_OS_LPTICK_TO_OSTICK(DEF_INT_16U_MAX_VAL);
            lpticks          = DEF_INT_16U_MAX_VAL;
        } else {
            BSP_OS_TicksToGo = ticks;
        }
                                                                /* Adjust for invalid value.                            */
        if (lpticks & 0xFFFF == 0) {
            lpticks          = DEF_INT_16U_MAX_VAL;
            BSP_OS_TicksToGo = BSP_OS_LPTICK_TO_OSTICK(DEF_INT_16U_MAX_VAL);
        }
    } else if (ticks == 0) {
        lpticks          = 1;
        BSP_OS_TicksToGo = 1;
    } else {
        lpticks          = DEF_INT_16U_MAX_VAL;
        BSP_OS_TicksToGo = 0;
    }


    RTC_Enable(false);                                          /* Stop timer.                                          */

                                                                /* Re-configure timer.                                  */

    RTC_CompareSet(0, lpticks);                                 /* Reset the RTC compare value to trigger at the        */
                                                                /* configured tick rate.                                */

    RTC_Enable(true);                                           /* Restart timer.                                       */

                                                                /* Return the the number of effective OS ticks that     */
    return (BSP_OS_TicksToGo);                                  /* will elapse before the next interrupt.               */
}


/*
*********************************************************************************************************
*                                          BSP_OS_LP_TickGet()
*
* Description : Gets the RTC timer state.
*
* Argument(s) : none.
*
* Return(s)   : RTC state.
*
* Caller(s)   : BSP_OS_TickNextSet and BSP_OS_TickGet.
*
*               This function is an INTERNAL uC/OS-III function & MUST NOT be called by application
*               function(s).
*
* Note(s)     : none.
*********************************************************************************************************
*/

OS_TICK  BSP_OS_LP_TickGet(void)
{
    OS_TICK  ticks;


    ticks = RTC_CounterGet();

    return (ticks);
}
#endif
